
local gr, pi, sin, cos, floor = love.graphics, math.pi, math.sin, math.cos, math.floor
local down = love.keyboard.isDown

function love.load()
	width, height	= 300, 300
	cw, ch			= width/2, height/2
	fov, vd			= 256, 400
	love.window.setMode(width,height,{borderless = true})
	gr.setBackgroundColor( 85, 85, 85, 0 )
	images = {'lenaC.jpg','blue.jpg','hand.jpg','pic.jpg','Bhutan.jpg','venus2.jpg'}
	texture = {}
	for i = 1, #images do texture[i] = gr.newImage(images[i]) end
	cube = {
		{-100, 100,-100}, {-100,-100,-100}, { 100,-100,-100}, { 100, 100,-100},
		{-100, 100, 100}, {-100,-100, 100}, { 100,-100, 100}, { 100, 100, 100} }
	faces = { {1,5,4,8}, {6,2,7,3}, {5,6,8,7}, {5,1,6,2}, {1,4,2,3}, {3,4,7,8} }
	tex = {{0,0},{0,1},{1,0},{1,1}}
	rotateX( cube, pi/4 )
	rotateY( cube, pi/4 )
	end

function love.update( dt )
	if down("left")  then rotateY(cube,  dt) end
	if down("right") then rotateY(cube, -dt) end
	if down("up") 	 then rotateX(cube, -dt) end
	if down("down")  then rotateX(cube,  dt) end
	if down('s')	 then vd = vd + 1 end
	if down('w')	 then vd = vd > 175 and vd - 1 or 175 end
	if down('d')	 then fov = fov + 1 end
	if down('a')	 then fov = fov - 1 end
	end

function love.draw() drawObject( cube, faces ) end

function love.keypressed( key )
	if key == 'escape'	then love.event.quit() end
	end

function drawObject( obj, faces )
	mesh = {}
	for i = 1, #faces do
		local pt = {}
		for j = 1, 4 do
			pt[j] = {floor(fov * obj[faces[i][j]][1] / (obj[faces[i][j]][3] + vd) + cw),
			   floor(fov * obj[faces[i][j]][2] / (obj[faces[i][j]][3] + vd) + ch),
			   tex[j][1],tex[j][2]}
			end
		mesh[i] = gr.newMesh(pt,texture[i],'strip')
		local sur = {}
		for j = 1, 3  do sur[j] = {obj[faces[i][j]][1],obj[faces[i][j]][2],obj[faces[i][j]][3]} end
		local nx,ny,nz = calc_norm( sur )
		if nz <-.33 then	-- don't draw back facing surfaces (Majic number - fov dependent)
			gr.draw(mesh[i],0,0)
			end
		end
	end

function rotateX( obj, a )
	local ca, sa = cos( a ), sin( a )
	for i = 1, #obj do
		obj[i][2], obj[i][3] = obj[i][2]*ca-obj[i][3]*sa, obj[i][2]*sa+obj[i][3]*ca
		end
	end

function rotateY( obj, a )
	local ca, sa = cos( a ), sin( a )
	for i = 1, #obj do
		obj[i][3], obj[i][1] = obj[i][3]*ca-obj[i][1]*sa, obj[i][3]*sa+obj[i][1]*ca
		end
	end

function calc_norm( sr )  -- surface normal from three points taken from a surface
	local a1, a2, a3 = sr[3][1] - sr[1][1], sr[3][2] - sr[1][2], sr[3][3] - sr[1][3]
	local b1, b2, b3 = sr[2][1] - sr[1][1], sr[2][2] - sr[1][2], sr[2][3] - sr[1][3]
	local n1, n2, n3 = a3*b2-a2 * b3, a1*b3-a3 * b1, a2*b1-a1 * b2 -- cross product
	local nl = math.sqrt( n1 * n1 + n2 * n2 + n3 * n3 )		-- normalize normals
	if nl == 0 then nl = 1 end
	return  n1 / nl, n2 / nl, n3 / nl
	end
